/*
 * Effect.java
 *
 * Last modified: <2008/08/29 17:35:53 +0900> By SUMIDA
 */
package game;

import java.util.Iterator;
import java.util.LinkedList;

public class Effect {

    public static final byte EFFECT_ID_EXPLOSION = 0;
    public static final byte EFFECT_ID_BOMB      = 1;
    public static final byte EFFECT_ID_FIRE      = 2;
    public static final byte EFFECT_ID_MOON      = 3;
    public static final byte EFFECT_ID_PLANET    = 4;
    public static final byte EFFECT_ID_HIT       = 5;
    public static final byte EFFECT_ID_MAX       = 6;

    public static class Info {
        public int id;
        public Sprite.Info sprite = new Sprite.Info();

        public void init() {
            id = 0;
            sprite.init();
        }
    }

    public static class Manager {
        LinkedList active = new LinkedList();
        LinkedList pool   = new LinkedList();
    }

    public static class Type {
        int tex_id;
        int options;
        int frame_start;
        int frame_end;
        int frame_interval;

        public Type(int tex_id, int options, int frame_start, int frame_end, int frame_interval) {
            this.tex_id = tex_id;
            this.options = options;
            this.frame_start = frame_start;
            this.frame_end = frame_end;
            this.frame_interval = frame_interval;
        }
    }

    private static Type[] types = {
        new Type(Texture.TEX_EXPLOSION, Sprite.SPRITE_FRAME64|Sprite.SPRITE_USE_ANIME, 0, 14, 1),
        new Type(Texture.TEX_BOMB     , Sprite.SPRITE_FRAME|Sprite.SPRITE_USE_ANIME  , 0, 15, 1),
        new Type(Texture.TEX_FIRE     , Sprite.SPRITE_USE_LOWER|Sprite.SPRITE_FRAME|Sprite.SPRITE_USE_ANIME|Sprite.SPRITE_USE_MOVE, 0,  9, 2),
        new Type(Texture.TEX_MOON     , Sprite.SPRITE_USE_MOVE|255, 0, 0, 0),
        new Type(Texture.TEX_PLANET   , Sprite.SPRITE_USE_MOVE|254, 0, 0, 0),
        new Type(Texture.TEX_HIT      , Sprite.SPRITE_USE_MOVE|Sprite.SPRITE_FRAME32|Sprite.SPRITE_USE_ANIME, 0, 7, 4),
    };

    private static Effect.Manager mgr = new Effect.Manager();

    public static void init() {

        int count = SystemInfo.EFFECT_MAX;
        if (mgr.active.isEmpty() && mgr.pool.isEmpty()) {
            while (count-- > 0) {
                mgr.pool.add(new Effect.Info());
            }

        } else {
            mgr.pool.addAll(mgr.active);
            mgr.active.clear();
        }

        for (Iterator it = mgr.pool.iterator(); it.hasNext();) {
            Info info = (Info)it.next();
            info.init();
        }
    }

    public static Info getInfo(int id) {

        Info info = (Info)mgr.pool.poll();

        if (info != null) {
            Type type = types[id];

            mgr.active.add(info);
            info.id                    = id;
            info.sprite.surface_id     = type.tex_id;
            info.sprite.options        = Sprite.SPRITE_VISIBLE|type.options;
            info.sprite.frame_start    = type.frame_start;
            info.sprite.frame_end      = type.frame_end;
            info.sprite.frame_interval = type.frame_interval;
        }

        return info;
    }

    public static void dropInfo(Info info) {

        info.init();
        mgr.pool.add(info);
    }

    public static void action() {

        LinkedList list = mgr.active;

        mgr.active = new LinkedList();

        while (!list.isEmpty()) {

            Info info = (Info)list.poll();

            if ((info.sprite.options & Sprite.SPRITE_STOP_ANIME) != 0) {
                dropInfo(info);
                continue;
            }

            {
                Surface.Info surface = Surface.getInfo(info.sprite.surface_id);

                int x0 = Real.RealToInt(info.sprite.x);
                int y0 = Real.RealToInt(info.sprite.y);
                int x1 = x0 + surface.frame_width  -1;
                int y1 = y0 + surface.frame_height -1;

                if (x0 > SystemInfo.CLIP_X1 || x1 < SystemInfo.CLIP_X0 || y0 > SystemInfo.CLIP_Y1 || y1 < SystemInfo.CLIP_Y0) {
                    dropInfo(info);
                    continue;
                }
            }

            Sprite.action(info.sprite);
            Sprite.put(info.sprite);
            mgr.active.add(info);
        }
    }

    public static void setPosition(Info info, Sprite.Info spr, int ox, int oy) {

        Surface.Info surface_effect = Surface.getInfo(info.sprite.surface_id);
        Surface.Info surface_ship = Surface.getInfo(spr.surface_id);

        info.sprite.x = spr.x + ox
            + (Real.IntToReal(surface_ship.frame_width - surface_effect.frame_width) >> 1);
        info.sprite.y = spr.y + oy
            + (Real.IntToReal(surface_ship.frame_height - surface_effect.frame_height) >> 1);
    }
}
