/*
 * Enemy.java
 *
 * Last modified: <2008/08/22 23:18:39 +0900> By SUMIDA
 */
package game;

import java.util.LinkedList;

public class Enemy {

    public static Ship.Manager enemy_mgr = new Ship.Manager();
    public static Shot.Manager enemy_shot_mgr = new Shot.Manager();

    private static EnemyModule[] func_enemy = {
        new Enemy1(),
        new Enemy2(),
        new Enemy3(),
        new Enemy4(),
        new Enemy5(),
        new Missile(),
        new Boss1(),
        new Boss2(),
    };

    public static void init() {
        Ship.init(enemy_mgr, SystemInfo.ENEMY_MAX);
        Shot.init(enemy_shot_mgr, SystemInfo.ENEMY_SHOT_MAX);
    }

    public static Ship.Info getInfo(int id, int x, int y) {

        Ship.Info info = Ship.getInfo(enemy_mgr, id);

        if (info != null) {
            info.stat     = Ship.SHIP_STAT_APPEAR;
            info.count    = 10;
            info.sprite.x = x;
            info.sprite.y = y;

            func_enemy[id - Ship.SHIP_ID_ENEMY].init(info);
        }

        return info;
    }

    public static void action() {

        LinkedList list = enemy_mgr.active;

        enemy_mgr.active = new LinkedList();

        while (!list.isEmpty()) {

            Ship.Info info = (Ship.Info)list.poll();

            if ((info.flag & Ship.SHIP_FLAG_NOCLIP) == 0) {
                Surface.Info surface = Surface.getInfo(info.sprite.surface_id);

                int x0 = Real.RealToInt(info.sprite.x);
                int y0 = Real.RealToInt(info.sprite.y);
                int x1 = x0 + surface.frame_width  -1;
                int y1 = y0 + surface.frame_height -1;

                if (x0 > SystemInfo.CLIP_X1 || x1 < SystemInfo.CLIP_X0 || y0 > SystemInfo.CLIP_Y1 || y1 < SystemInfo.CLIP_Y0) {
                    Logging.trace("delete(info=" + info + ", id=" + info.id + ", x0=" + x0 + ", y0=" + y0 + ", x1=" + x1 + ", y1=" + y1 +")");
                    Ship.dropInfo(enemy_mgr, info);
                    continue;
                }
            }

            /* 動作カウンタのデクリメント */
            --info.count;

            /* 動作状態毎の処理を実行 */
            switch (info.stat) {
            case Ship.SHIP_STAT_APPEAR:
            case Ship.SHIP_STAT_NORMAL:
            case Ship.SHIP_STAT_MOVE:
            case Ship.SHIP_STAT_FIRE:
                if (Sprite.collisionCheck(info.sprite, MyShip.ship.sprite)) {
                    Ship.down(MyShip.ship, info, info.sprite.dx >> 2, info.sprite.dy >> 2);
                }

                if (info.stat != Ship.SHIP_STAT_EXPLOSION) {
                    func_enemy[info.id - Ship.SHIP_ID_ENEMY].action(info);
                }
                break;

            case Ship.SHIP_STAT_EXPLOSION:
                if (info.count < 0) {
                    info.stat = Ship.SHIP_STAT_LOST;
                }
                break;

            default:
                break;
            }

            if (info.stat == Ship.SHIP_STAT_LOST) {
                /* 削除 */
                func_enemy[info.id - Ship.SHIP_ID_ENEMY].term(info);
                Logging.trace("delete(info=" + info + ", id=%" + info.id + ")");
                Ship.dropInfo(enemy_mgr, info);
                continue;
            }

            /* 被弾状態を解除  */
            info.flag &= ~Ship.SHIP_FLAG_DOWN;

            /* 移動 */
            Sprite.action(info.sprite);

            /* 表示 */
            Sprite.put(info.sprite);

            /* 再登録 */
            enemy_mgr.active.add(info);
        }
    }
}
