/*
 * Logging.java
 *
 * Last modified: <2008/08/22 06:57:54 +0900> By SUMIDA
 */
package game;

import java.text.SimpleDateFormat;
import java.util.Calendar;

/**
 * ログ出力を管理するクラス。
 * 本来であれば、java.util.loggingのAPIに委譲すべきだが…
 * 制限が厳しいので自前かつcommons logging似のメソッドを用意した。
 */
public class Logging {

    private static SimpleDateFormat sdf = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss.SSS - ");

    public static boolean isTraceEnable() {
        return ((SystemInfo.options & SystemInfo.SYSTEM_OPT_VERBOSE) != 0) ? true: false;
    }

    public static void info(String str) {
        Calendar now = Calendar.getInstance();
        StackTraceElement e = new Exception().getStackTrace()[1];
        String header = sdf.format(now.getTime()) + e.getClassName() + "." + e.getMethodName() + "(" + e.getLineNumber() + ") [INFO ] ";
        System.out.println(header + str);
    }

    public static void warn(String str) {
        if (isTraceEnable()) {
            Calendar now = Calendar.getInstance();
            StackTraceElement e = new Exception().getStackTrace()[1];
            String header = sdf.format(now.getTime()) + e.getClassName() + "." + e.getMethodName() + "(" + e.getLineNumber() + ") [WARN ] ";
            System.out.println(header + str);
        }
    }

    public static void trace(String str) {
        if (isTraceEnable()) {
            Calendar now = Calendar.getInstance();
            StackTraceElement e = new Exception().getStackTrace()[1];
            String header = sdf.format(now.getTime()) + e.getClassName() + "." + e.getMethodName() + "(" + e.getLineNumber() + ") [TRACE] ";
            System.out.println(header + str);
        }
    }

    public static void error(String str) {
        Calendar now = Calendar.getInstance();
        StackTraceElement e = new Exception().getStackTrace()[1];
        String header = sdf.format(now.getTime()) + e.getClassName() + "." + e.getMethodName() + "(" + e.getLineNumber() + ") [ERROR] ";
        System.err.println(header + str);
    }

    public static void error(String str, Throwable th) {
        Calendar now = Calendar.getInstance();
        StackTraceElement e = new Exception().getStackTrace()[1];
        String header = sdf.format(now.getTime()) + e.getClassName() + "." + e.getMethodName() + "(" + e.getLineNumber() + ") [ERROR] ";
        System.err.println(header + str);
        System.err.println(th.getMessage());
        th.printStackTrace();
        //StackTraceElement[] array = th.getStackTrace();
        //for (StackTraceElement element: array) {
        //    System.out.println(element.toString());
        //}
    }
}
