/*
 * Ship.java
 *
 * Last modified: <2008/08/29 16:03:02 +0900> By SUMIDA
 */
package game;

import java.util.LinkedList;

public class Ship {

    public static final byte SHIP_DEF_LOST_WAIT = 2;

    public static final byte SHIP_ID_ME     = 0;
    public static final byte SHIP_ID_ENEMY  = 1;
    public static final byte SHIP_ID_ENEMY1 = SHIP_ID_ENEMY;
    public static final byte SHIP_ID_ENEMY2 = 2;
    public static final byte SHIP_ID_ENEMY3 = 3;
    public static final byte SHIP_ID_ENEMY4 = 4;
    public static final byte SHIP_ID_ENEMY5 = 5;
    public static final byte SHIP_ID_MISSILE = 6;
    public static final byte SHIP_ID_BOSS1 = 7;
    public static final byte SHIP_ID_BOSS2 = 8;
    public static final byte SHIP_ID_MAX   = 9;

    public static final byte SHIP_STAT_LOST      = 0; /* 消滅     */
    public static final byte SHIP_STAT_APPEAR    = 1; /* 出現     */
    public static final byte SHIP_STAT_NORMAL    = 2; /* 通常状態 */
    public static final byte SHIP_STAT_MOVE      = 3; /* 移動     */
    public static final byte SHIP_STAT_FIRE      = 4; /* 砲撃     */
    public static final byte SHIP_STAT_EXPLOSION = 5; /* 爆発     */
    public static final byte SHIP_STAT_MAX       = 6;

    public static final int SHIP_FLAG_DOWN     = 0x01000000;     /* 被弾               */
    public static final int SHIP_FLAG_HEAVY    = 0x02000000;     /* 重量大             */
    public static final int SHIP_FLAG_NODAMAGE = 0x04000000;     /* ノーダメージ       */
    public static final int SHIP_FLAG_NOCLIP   = 0x08000000;     /* 画面外消滅なし     */
    public static final int SHIP_FLAG_NOPOOL   = 0x10000000;     /* プール制御なし     */
    public static final int SHIP_FLAG_SYSTEM   = 0xff000000;     /* システムフラグ域   */
    public static final int SHIP_FLAG_USERS = ~SHIP_FLAG_SYSTEM; /* ユーザ利用フラグ域 */

    private static final int SPRITE_OPT_SHIP  = Sprite.SPRITE_VISIBLE|Sprite.SPRITE_FRAME|Sprite.SPRITE_LOOP_ANIME|Sprite.SPRITE_USE_ANIME|Sprite.SPRITE_USE_MOVE|Sprite.SPRITE_USE_COLLISION;
    private static final int SHOT_LIMIT_STRENGTH = 10;

    public static class Info implements StrengthUsable {

        public int id;
        public Sprite.Info sprite = new Sprite.Info();
        public int stat;
        public int flag;
        public int count;
        public int point;
        public int strength;

        public int getStrength() {return strength;}
        public void setStrength(int value) {strength = value;}
    }

    public static class Manager {
        LinkedList active = new LinkedList();
        LinkedList pool   = new LinkedList();
    }

    public static class Type {
        int tex_id;
        int strength;
        int point;
        int options;
        int collision_x0;
        int collision_y0;
        int collision_x1;
        int collision_y1;

        public Type(
            int tex_id,
            int strength,
            int point,
            int options,
            int collision_x0,
            int collision_y0,
            int collision_x1,
            int collision_y1) {

            this.tex_id = tex_id;
            this.strength = strength;
            this.point = point;
            this.options = options;
            this.collision_x0 = collision_x0;
            this.collision_y0 = collision_y0;
            this.collision_x1 = collision_x1;
            this.collision_y1 = collision_y1;
        }
    }

    private static Type[] types = {
        new Type(Texture.TEX_SHIP  ,     1,    0,  2|SPRITE_OPT_SHIP , Real.IntToReal(28), Real.IntToReal(10), Real.IntToReal(60-29) , Real.IntToReal(60-41)),
        new Type(Texture.TEX_ENEMY1,     1,   10, 10|SPRITE_OPT_SHIP , Real.IntToReal(2) , Real.IntToReal(2) , Real.IntToReal(60-3)  , Real.IntToReal(60-3) ),
        new Type(Texture.TEX_ENEMY2,     2,   20, 11|SPRITE_OPT_SHIP , Real.IntToReal(2) , Real.IntToReal(2) , Real.IntToReal(60-3)  , Real.IntToReal(60-3) ),
        new Type(Texture.TEX_ENEMY3,    10,   30, 12|SPRITE_OPT_SHIP , Real.IntToReal(2) , Real.IntToReal(2) , Real.IntToReal(60-3)  , Real.IntToReal(60-3) ),
        new Type(Texture.TEX_ENEMY4,    20,   40, 13|SPRITE_OPT_SHIP , Real.IntToReal(2) , Real.IntToReal(2) , Real.IntToReal(60-3)  , Real.IntToReal(60-3) ),
        new Type(Texture.TEX_ENEMY5,    30,   50, 14|SPRITE_OPT_SHIP , Real.IntToReal(2) , Real.IntToReal(2) , Real.IntToReal(60-3)  , Real.IntToReal(60-3) ),
        new Type(Texture.TEX_MISSILE,    1,   10, 15|SPRITE_OPT_SHIP , Real.IntToReal(4) , Real.IntToReal(10), Real.IntToReal(16-4)  , Real.IntToReal(54-1) ),
        new Type(Texture.TEX_BOSS1 ,   500, 1000, 20|SPRITE_OPT_SHIP , Real.IntToReal(18), Real.IntToReal(45), Real.IntToReal(100-19), Real.IntToReal(164)  ),
        new Type(Texture.TEX_BOSS2 ,  1000, 5000, 20|SPRITE_OPT_SHIP , Real.IntToReal(8) , Real.IntToReal(42), Real.IntToReal(100-9) , Real.IntToReal(116)  ),
    };

    public static void init(Manager mgr, int count) {

        if (mgr.active.isEmpty() && mgr.pool.isEmpty()) {
            while (count-- > 0) {
                mgr.pool.add(new Info());
            }

        } else {
            mgr.pool.addAll(mgr.active);
            mgr.active.clear();
        }
    }


    public static Info getInfo(Manager mgr, int id) {

        Info info = (Info)mgr.pool.poll();

        if (info != null) {
            Type type = types[id];

            mgr.active.add(info);
            info.id       = id;
            info.stat     = SHIP_STAT_NORMAL;
            info.flag     = 0;
            info.count    = 0;
            info.point    = type.point;
            info.strength = type.strength;
            info.sprite.surface_id = type.tex_id;
            info.sprite.options = type.options;
            info.sprite.frame_start    = 0;
            info.sprite.frame_end      = 1;
            info.sprite.frame_interval = 5;
            info.sprite.collision_x0   = type.collision_x0;
            info.sprite.collision_y0   = type.collision_y0;
            info.sprite.collision_x1   = type.collision_x1;
            info.sprite.collision_y1   = type.collision_y1;
        }

        return info;
    }

    public static void dropInfo(Manager mgr, Info info) {

        mgr.pool.add(info);
    }

    public static int down(Info info, StrengthUsable ctr, int dx, int dy) {

        int score = 0;

        if (info.stat != Ship.SHIP_STAT_EXPLOSION && info.stat != Ship.SHIP_STAT_LOST) {

            int strength = ctr.getStrength();
            int pow = (strength > SHOT_LIMIT_STRENGTH)? SHOT_LIMIT_STRENGTH: strength;

            strength -= pow;
            ctr.setStrength(strength);
            if ((info.flag & SHIP_FLAG_NODAMAGE) == 0) {
                info.strength -= pow;
            }

            if (info.strength > 0) {
                info.flag |= SHIP_FLAG_DOWN;
                if ((info.flag & SHIP_FLAG_HEAVY) == 0) {
                    info.sprite.x += dx;
                    info.sprite.y += dy;
                }
                Sound.playEffect(Audio.SE_SHOT1);

            } else {
                Effect.Info effect = null;

                //System.out.println(String.format("bomb! (info=%s, id=%d, point=%d)", info, info.id, info.point));
                score = info.point;
                info.stat = Ship.SHIP_STAT_EXPLOSION;
                info.count = SHIP_DEF_LOST_WAIT;
                info.sprite.options &= ~Sprite.SPRITE_USE_MOVE;

                switch (info.id) {
                case SHIP_ID_BOSS1:
                case SHIP_ID_BOSS2:
                    effect = Effect.getInfo(Effect.EFFECT_ID_BOMB);
                    Sound.playEffect(Audio.SE_BOMB1);
                    break;
                default:
                    effect = Effect.getInfo(Effect.EFFECT_ID_EXPLOSION);
                    Sound.playEffect(Audio.SE_BOMB0);
                    break;
                }

                if (effect != null) {
                    Effect.setPosition(effect, info.sprite, 0, 0);
                }
            }
        }

        return score;
    }
}
