/*
 * Shot.java
 *
 * Last modified: <2008/08/29 17:33:49 +0900> By SUMIDA
 */
package game;

import java.util.Iterator;
import java.util.LinkedList;

public class Shot {
    public static final byte SHOT_LIMIT_STRENGTH = 10;

    public static final byte SHOT_ID_NORMAL = 0;
    public static final byte SHOT_ID_BALLET = 1;
    public static final byte SHOT_ID_RAY1   = 2;
    public static final byte SHOT_ID_RAY2   = 3;
    public static final byte SHOT_ID_PHOTON = 4;
    public static final byte SHOT_ID_MAX    = 5;

    public static final byte SHOT_STAT_LOST      = 0;      /* 消滅     */
    public static final byte SHOT_STAT_NORMAL    = 1;      /* 通常状態 */
    public static final byte SHOT_STAT_SEARCH    = 2;      /* 探索状態 */
    public static final byte SHOT_STAT_EXPLOSION = 3;      /* 爆発     */
    public static final byte SHOT_STAT_MAX       = 4;

    public static class Info implements StrengthUsable {

        public int id;
        public Sprite.Info sprite = new Sprite.Info();
        public int stat;
        public int count;
        public int strength;

        public void init() {
            id       = 0;
            stat     = 0;
            count    = 0;
            strength = 0;
            sprite.init();
        }

        public int getStrength() {return strength;}
        public void setStrength(int value) {strength = value;}
    }

    public static class Manager {
        LinkedList active = new LinkedList();
        LinkedList pool   = new LinkedList();
    }

    public static class Type {
        int tex_id;
        int strength;
        int options;
        int frame_start;
        int frame_end;
        int frame_interval;

        public Type(int tex_id, int strength, int options, int frame_start, int frame_end, int frame_interval) {
            this.tex_id = tex_id;
            this.strength = strength;
            this.options = options;
            this.frame_start = frame_start;
            this.frame_end = frame_end;
            this.frame_interval = frame_interval;
        }
    }

    private static Type[] types = {
        new Type(Texture.TEX_SHOT  ,  1,   1|Sprite.SPRITE_FRAME, 0, 0, 0),
        new Type(Texture.TEX_BALLET,  1,   1|Sprite.SPRITE_FRAME8|Sprite.SPRITE_USE_ANIME|Sprite.SPRITE_LOOP_ANIME, 0, 3, 2),
        new Type(Texture.TEX_RAY1  ,  2,   1|Sprite.SPRITE_FRAME, 0, 0, 0),
        new Type(Texture.TEX_RAY2  ,  5, 100|Sprite.SPRITE_FRAME, 0, 0, 0),
        new Type(Texture.TEX_PHOTON, 10,   1|Sprite.SPRITE_FRAME|Sprite.SPRITE_USE_ANIME|Sprite.SPRITE_LOOP_ANIME, 0, 3, 8),
    };

    public static void init(Shot.Manager mgr, int count) {

        if (mgr.active.isEmpty() && mgr.pool.isEmpty()) {
            while (count-- > 0) {
                mgr.pool.add(new Shot.Info());
            }

        } else {
            mgr.pool.addAll(mgr.active);
            mgr.active.clear();
        }

        for (Iterator it = mgr.pool.iterator(); it.hasNext();) {
            Shot.Info info = (Shot.Info)it.next();
            info.init();
        }
    }

    public static Shot.Info getInfo(Shot.Manager mgr, int id) {

        Shot.Info info = (Shot.Info)mgr.pool.poll();

        if (info != null) {
            Type type = types[id];
            Surface.Info surface = Surface.getInfo(type.tex_id);

            mgr.active.add(info);
            info.id       = id;
            info.stat     = SHOT_STAT_NORMAL;
            info.count    = 0;
            info.strength = type.strength;
            info.sprite.surface_id = type.tex_id;
            info.sprite.options = Sprite.SPRITE_VISIBLE|Sprite.SPRITE_USE_MOVE|Sprite.SPRITE_USE_COLLISION|type.options;
            info.sprite.frame_start    = type.frame_start;
            info.sprite.frame_end      = type.frame_end;
            info.sprite.frame_interval = type.frame_interval;
            info.sprite.collision_x0 = Real.IntToReal(0);
            info.sprite.collision_y0 = Real.IntToReal(0);
            info.sprite.collision_x1 = Real.IntToReal(surface.frame_width - 1);
            info.sprite.collision_y1 = Real.IntToReal(surface.frame_height - 1);
        }

        return info;
    }

    public static void dropInfo(Shot.Manager mgr, Shot.Info info) {

        info.init();
        mgr.pool.add(info);
    }

    public static int action(Shot.Manager mgr, Ship.Manager ship_mgr) {

        LinkedList list = mgr.active;
        mgr.active = new LinkedList();
        int score = 0;

        while (!list.isEmpty()) {

            Shot.Info info = (Shot.Info)list.poll();
            Sprite.Info sprite = info.sprite;

            /* ショット毎の特殊処理 */
            switch (info.id) {
            case SHOT_ID_PHOTON:
                photon(info, ship_mgr.active.iterator());
                break;
            default:
                break;
            }

            /* はみ出した場合は削除 */
            {
                Surface.Info surface = Surface.getInfo(sprite.surface_id);
                int x0 = Real.RealToInt(sprite.x);
                int y0 = Real.RealToInt(sprite.y);
                int x1 = x0 + surface.frame_width  -1;
                int y1 = y0 + surface.frame_height -1;

                if (x0 > SystemInfo.CLIP_X1 || x1 < SystemInfo.CLIP_X0 || y0 > SystemInfo.CLIP_Y1 || y1 < SystemInfo.CLIP_Y0) {
                    dropInfo(mgr, info);
                    continue;
                }
            }

            /* 敵機との接触判定 */
            Iterator it_ship = ship_mgr.active.iterator();
            while (it_ship.hasNext()) {
                Ship.Info ship = (Ship.Info)it_ship.next();

                if (ship.stat != Ship.SHIP_STAT_LOST
                    && ship.stat != Ship.SHIP_STAT_EXPLOSION
                    && Sprite.collisionCheck(sprite, ship.sprite)) {

                    //trace("hit!");
                    score += Ship.down(ship, info, sprite.dx >> 3, sprite.dy >> 3);

                    if (info.strength <= 0) {
                        if (info.id == SHOT_ID_NORMAL) {
                            Effect.Info effect = Effect.getInfo(Effect.EFFECT_ID_HIT);

                            if (effect != null) {
                                Effect.setPosition(effect, info.sprite, 0, 0);
                                effect.sprite.dx = Real.ToReal(SystemInfo.Random(-1.5, 1.5));
                                effect.sprite.dy = Real.ToReal(SystemInfo.Random(0, 2));
                                info.count = 5;
                            }
                        }

                        dropInfo(mgr, info);
                        sprite = null;
                        break;
                    }
                }
            }

            /* 非接触時のみ表示以下を行う */
            if (sprite != null) {
                /* ショットの移動処理 */
                Sprite.action(sprite);

                /* ショットの表示 */
                Sprite.put(sprite);

                /* 再登録 */
                mgr.active.add(info);
            }
            //trace(String.format("shot.sprite: id=%d, options=%08x, frame_counter=%d", sprite.surface_id, sprite.options, sprite.frame_counter));
        }

        //trace(String.format("shot=%d", mgr.pool.size() + mgr.active.size()));

        return score;
    }

    public static void fire(Shot.Info info, Ship.Info ship, int ox, int oy) {

        Surface.Info surface_shot = Surface.getInfo(info.sprite.surface_id);
        Surface.Info surface_ship = Surface.getInfo(ship.sprite.surface_id);

        info.sprite.x = ship.sprite.x + ox
            + (Real.IntToReal(surface_ship.frame_width - surface_shot.frame_width) >> 1);
        info.sprite.y = ship.sprite.y + oy
            + (Real.IntToReal(surface_ship.frame_height - surface_shot.frame_height) >> 1);
    }

    public static void homing(Shot.Info info, Ship.Info ship, int speed) {

        Surface.Info surface_ship = Surface.getInfo(ship.sprite.surface_id);
        Surface.Info surface_shot = Surface.getInfo(info.sprite.surface_id);
        double dx = (ship.sprite.x + (Real.IntToReal(surface_ship.frame_width) >> 1)
                     - info.sprite.x - (Real.IntToReal(surface_shot.frame_width) >> 1)) / 65536.0;
        double dy = (ship.sprite.y + (Real.IntToReal(surface_ship.frame_height) >> 1)
                     - info.sprite.y - (Real.IntToReal(surface_shot.frame_height) >> 1)) / 65536.0;
        double d = Math.sqrt(dx * dx + dy * dy);

        if (d != 0) {
            double spd = speed / 65536.0;
            info.sprite.dx = (int)((dx / d * spd) * 65536);
            info.sprite.dy = (int)((dy / d * spd) * 65536);
        } else {
            info.sprite.dx = 0;
            info.sprite.dy = speed;
        }
    }

    /* ホーミング光子魚雷 */
    private static void photon(Shot.Info info, Iterator it) {

        if (it.hasNext()) {
            Ship.Info ship = (Ship.Info)it.next();

            if (info.count++ < 60 * 3) {

                int dx = info.sprite.dx;
                int dy = info.sprite.dy;

                homing(info, ship, Real.IntToReal(1));

                info.sprite.dx = (info.sprite.dx + dx * 7) >> 3;
                info.sprite.dy = (info.sprite.dy + dy * 7) >> 3;

            } else if((info.count & 0x03) == 0) {
                if (Real.RealMul(info.sprite.dx, info.sprite.dx)
                    + Real.RealMul(info.sprite.dy, info.sprite.dy) < Real.IntToReal(16)) {
                    info.sprite.dx += info.sprite.dx >> 4;
                    info.sprite.dy += info.sprite.dy >> 4;
                }
            }
        }
    }
}
